@@ -1,5 +1,6 @@
 # -*- coding: utf-8 -*-
 from Acquisition import aq_base
+from collective.exportimport import config
 from OFS.interfaces import IOrderedContainer
 from operator import itemgetter
 from plone import api
@@ -24,7 +25,6 @@ from plone.portlets.constants import (
     CONTENT_TYPE_CATEGORY,
     CONTEXT_CATEGORY,
 )
-from z3c.relationfield import RelationValue
 from zope.component import getUtilitiesFor
 from zope.component import queryMultiAdapter
 from zope.interface import providedBy
@@ -49,6 +49,13 @@ except pkg_resources.DistributionNotFound:
 else:
     HAS_DX = True
 
+try:
+    pkg_resources.get_distribution("z3c.relationfield")
+except pkg_resources.DistributionNotFound:
+    RelationValue = None
+else:
+    from z3c.relationfield import RelationValue
+
 
 logger = logging.getLogger(__name__)
 
@@ -61,8 +68,7 @@ class ExportRelations(BrowserView):
         if not self.request.form.get("form.submitted", False):
             return self.index()
 
-        self.debug = debug
-        all_stored_relations = self.get_all_references()
+        all_stored_relations = self.get_all_references(debug)
         data = json.dumps(all_stored_relations, indent=4)
         filename = "relations.json"
         self.request.response.setHeader("Content-type", "application/json")
@@ -72,7 +78,7 @@ class ExportRelations(BrowserView):
         )
         return self.request.response.write(safe_bytes(data))
 
-    def get_all_references(self):
+    def get_all_references(self, debug=False):
         results = []
 
         if HAS_AT:
@@ -94,9 +100,12 @@ class ExportRelations(BrowserView):
                         "to_uuid": rel.targetUID,
                         "relationship": rel.relationship,
                     }
-                    if self.debug:
+                    if debug:
                         item["from_path"] = source.absolute_url_path()
                         item["to_path"] = target.absolute_url_path()
+                    item = self.reference_hook(item)
+                    if item is None:
+                        continue
                     results.append(item)
 
         if HAS_DX:
@@ -122,10 +131,16 @@ class ExportRelations(BrowserView):
                             if self.debug:
                                 item["from_path"] = from_brain[0].getPath()
                                 item["to_path"] = to_brain[0].getPath()
+                            item = self.reference_hook(item)
+                            if item is None:
+                                continue
                             results.append(item)
 
         return results
 
+    def reference_hook(self, item):
+        return item
+
 
 class ExportMembers(BrowserView):
     """Export plone groups and members"""
@@ -341,30 +356,36 @@ class ExportLocalRoles(BrowserView):
         return self.request.response.write(safe_bytes(data))
 
     def all_localroles(self):
-        results = []
-
-        def get_localroles(obj, path):
-            uid = IUUID(obj, None)
-            if not uid:
-                return
-            localroles = None
-            block = None
-            obj = aq_base(obj)
-            if getattr(obj, "__ac_local_roles__", None) is not None:
-                localroles = obj.__ac_local_roles__
-            if getattr(obj, "__ac_local_roles_block__", False):
-                block = obj.__ac_local_roles_block__
-            if localroles or block:
-                item = {"uuid": uid}
-                if localroles:
-                    item["localroles"] = localroles
-                if block:
-                    item["block"] = 1
-                results.append(item)
+        self.results = []
 
         portal = api.portal.get()
-        portal.ZopeFindAndApply(portal, search_sub=True, apply_func=get_localroles)
-        return results
+        portal.ZopeFindAndApply(portal, search_sub=True, apply_func=self.get_localroles)
+        return self.results
+
+    def get_localroles(self, obj, path):
+        uid = IUUID(obj, None)
+        if not uid:
+            return
+        localroles = None
+        block = None
+        obj = aq_base(obj)
+        if getattr(obj, "__ac_local_roles__", None) is not None:
+            localroles = obj.__ac_local_roles__
+        if getattr(obj, "__ac_local_roles_block__", False):
+            block = obj.__ac_local_roles_block__
+        if localroles or block:
+            item = {"uuid": uid}
+            if localroles:
+                item["localroles"] = localroles
+            if block:
+                item["block"] = 1
+            item = self.item_hook(item)
+            if item is None:
+                return
+            self.results.append(item)
+
+    def item_hook(self, item):
+        return item
 
 
 class ExportOrdering(BrowserView):
@@ -439,6 +460,9 @@ class ExportDefaultPages(BrowserView):
 
         portal = api.portal.get()
         portal.ZopeFindAndApply(portal, search_sub=True, apply_func=get_default_page)
+        portal_default_page = portal.getDefaultPage()
+        if portal_default_page:
+            results.append({"uuid": config.SITE_ROOT, "default_page": portal_default_page})
         return results
 
 
@@ -545,7 +569,7 @@ def export_local_portlets(obj):
             values = {}
             for name in schema.names():
                 value = getattr(assignment, name, None)
-                if isinstance(value, RelationValue):
+                if RelationValue is not None and isinstance(value, RelationValue):
                     value = value.to_object.UID()
                 elif isinstance(value, RichTextValue):
                     value = {
