@@ -8,10 +8,14 @@ from plone.i18n.normalizer.interfaces import IIDNormalizer
 from plone.namedfile.file import NamedBlobFile
 from plone.namedfile.file import NamedBlobImage
 from plone.restapi.interfaces import IDeserializeFromJson
+from Products.CMFPlone.interfaces.constrains import ISelectableConstrainTypes
+from Products.CMFPlone.interfaces.constrains import ENABLED
+from Products.CMFPlone.utils import _createObjectByType
 from Products.Five import BrowserView
 from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
 from six.moves.urllib.parse import unquote
 from six.moves.urllib.parse import urlparse
+from zExceptions import NotFound
 from zope.component import getMultiAdapter
 from zope.component import getUtility
 from ZPublisher.HTTPRequest import FileUpload
@@ -88,13 +92,27 @@ class ImportContent(BrowserView):
     DEFAULTS = {}
 
     def __call__(self, jsonfile=None, return_json=False, limit=None, server_file=None):
+        request = self.request
         self.limit = limit
+        self.commit = int(request["commit"]) if request.get("commit") else None
+        self.import_to_current_folder = request.get("import_to_current_folder", False)
+
+        self.handle_existing_content = int(request.get("handle_existing_content", 0))
+        self.handle_existing_content_options = (
+            ("0", "Skip: Don't import at all"),
+            ("1", "Replace: Delete item and create new"),
+            ("2", "Update: Reuse and only overwrite imported data"),
+            ("3", "Ignore: Create with a new id"),
+        )
 
         if not self.request.form.get("form.submitted", False):
             return self.template()
 
         # If we open a server file, we should close it at the end.
         close_file = False
+        status = "success"
+        msg = ""
+
         if server_file and jsonfile:
             # This is an error.  But when you upload 10 GB AND select a server file,
             # it is a pity when you would have to upload again.
@@ -104,6 +122,7 @@ class ImportContent(BrowserView):
                 type="warn",
             )
             server_file = None
+            status = "error"
         if server_file and not jsonfile:
             if server_file in self.server_files:
                 for path in self.import_paths:
@@ -115,13 +134,13 @@ class ImportContent(BrowserView):
                         close_file = True
                         break
             else:
+                msg = "File '{}' not found on server.".format(server_file)
                 api.portal.show_message(
-                    u"File not found on server.", request=self.request, type="warn"
-                )
+                    msg, request=self.request, type="warn")
                 server_file = None
+                status = "error"
         if jsonfile:
             self.portal = api.portal.get()
-            status = "success"
             try:
                 if isinstance(jsonfile, str):
                     return_json = True
@@ -159,6 +178,13 @@ class ImportContent(BrowserView):
     def finish(self):
         """Hook to do something after importing one file."""
 
+    def commit_hook(self, added, index):
+        """Hook to do something after importing every x items."""
+        msg = u"Committing after creating {} of {} handled items...".format(len(added), index)
+        logger.info(msg)
+        transaction.get().note(msg)
+        transaction.commit()
+
     @property
     def import_paths(self):
         # Adapted from ObjectManager.list_imports, which lists zexps.
@@ -220,6 +246,7 @@ class ImportContent(BrowserView):
             for drop in self.DROP_PATHS:
                 if drop in item["@id"]:
                     skip = True
+                    logger.info(u"Skipping {}".format(item['@id']))
             if skip:
                 continue
 
@@ -261,35 +288,72 @@ class ImportContent(BrowserView):
                 )
                 continue
 
-            # Speed up import by not using autogenerated ids for conflicts
-            if new_id in container:
-                duplicate = new_id
-                new_id = "{}-{}".format(new_id, random.randint(1000, 9999))
-                item["id"] = new_id
-                logger.info(
-                    u"{} ({}) already exists. Created as {}".format(
-                        duplicate, item["@id"], new_id
-                    )
-                )
-
             factory_kwargs = item.get("factory_kwargs", {})
-            container.invokeFactory(item["@type"], item["id"], **factory_kwargs)
-            new = container[item["id"]]
+
+            # Handle existing content
+            self.update_existing = False
+            if new_id in container:
+                if self.handle_existing_content == 0:
+                    # Skip
+                    logger.info(u"{} ({}) already exists. Skipping it.".format(
+                        new_id, item["@id"])
+                    )
+                    continue
+
+                elif self.handle_existing_content == 1:
+                    # Replace content before creating it new
+                    logger.info(u"{} ({}) already exists. Replacing it.".format(
+                        new_id, item["@id"])
+                    )
+                    api.content.delete(container[new_id])
+
+                elif self.handle_existing_content == 2:
+                    # Update existing item
+                    logger.info(u"{} ({}) already exists. Updating it.".format(
+                        new_id, item["@id"])
+                    )
+                    self.update_existing = True
+                    new = container[new_id]
+
+                else:
+                    # Create with new id. Speed up by using random id.
+                    duplicate = new_id
+                    new_id = "{}-{}".format(new_id, random.randint(1000, 9999))
+                    item["id"] = new_id
+                    logger.info(
+                        u"{} ({}) already exists. Created as {}".format(
+                            duplicate, item["@id"], new_id
+                        )
+                    )
+
+            if not self.update_existing:
+                # create without checking constrains and permissions
+                new = _createObjectByType(item["@type"], container, item["id"], **factory_kwargs)
+
+            new, item = self.global_obj_hook_before_deserializing(new, item)
 
             # import using plone.restapi deserializers
             deserializer = getMultiAdapter((new, self.request), IDeserializeFromJson)
-            new = deserializer(validate_all=False, data=item)
+            try:
+                new = deserializer(validate_all=False, data=item)
+            except Exception as error:
+                logger.warning(
+                    "cannot deserialize {}: {}".format(item["@id"], repr(error))
+                )
+                continue
 
             # Blobs can be exported as only a path in the blob storage.
             # It seems difficult to dynamically use a different deserializer,
             # based on whether or not there is a blob_path somewhere in the item.
             # So handle this case with a separate method.
             self.import_blob_paths(new, item)
+            self.import_constrains(new, item)
 
             self.global_obj_hook(new, item)
             self.custom_obj_hook(new, item)
 
             uuid = self.set_uuid(item, new)
+
             if uuid != item["UID"]:
                 item["UID"] = uuid
 
@@ -308,9 +372,7 @@ class ImportContent(BrowserView):
                 try:
                     modified_data = datetime.strptime(modified, "%Y-%m-%dT%H:%M:%S%z")
                 except ValueError:
-                    modified_data = datetime.strptime(
-                        modified[:-6], "%Y-%m-%dT%H:%M:%S"
-                    )
+                    modified_data = datetime.strptime(modified[:19], "%Y-%m-%dT%H:%M:%S")
                 modification_date = DateTime(modified_data)
                 new.modification_date = modification_date
                 new.modification_date_migrated = modification_date
@@ -320,12 +382,16 @@ class ImportContent(BrowserView):
                 try:
                     created_data = datetime.strptime(created, "%Y-%m-%dT%H:%M:%S%z")
                 except Exception:
-                    created_data = datetime.strptime(created[:-6], "%Y-%m-%dT%H:%M:%S")
+                    created_data = datetime.strptime(created[:19], "%Y-%m-%dT%H:%M:%S")
                 creation_date = DateTime(created_data)
                 new.creation_date = creation_date
                 new.creation_date_migrated = creation_date
-            logger.info("Created {} {}".format(item["@type"], new.absolute_url()))
+            logger.info("Created item #{}: {} {}".format(index, item["@type"], new.absolute_url()))
             added.append(new.absolute_url())
+
+            if self.commit and not len(added) % self.commit:
+                self.commit_hook(added, index)
+
         return added
 
     def handle_broken(self, item):
@@ -414,8 +480,32 @@ class ImportContent(BrowserView):
             item = modifier(item)
         return item
 
+    def import_constrains(self, obj, item):
+        if not item.get("exportimport.constrains"):
+            return
+        constrains = ISelectableConstrainTypes(obj, None)
+        if constrains is None:
+            return
+        constrains.setConstrainTypesMode(ENABLED)
+        locally_allowed_types = item["exportimport.constrains"]["locally_allowed_types"]
+        constrains.setLocallyAllowedTypes(locally_allowed_types)
+        immediately_addable_types = item["exportimport.constrains"]["immediately_addable_types"]
+        constrains.setImmediatelyAddableTypes(immediately_addable_types)
+
+    def global_obj_hook_before_deserializing(self, obj, item):
+        """Hook to modify the created obj before deserializing the data.
+        Example that applies marker-interfaces:
+
+        for iface_name in item.pop("marker_interfaces", []):
+            iface = resolveDottedName(iface_name)
+            if not iface.providedBy(obj):
+                alsoProvides(obj, iface)
+        return obj, item
+        """
+        return obj, item
+
     def global_obj_hook(self, obj, item):
-        """Override hook to modify all items of the imported item by type."""
+        """Override hook to modify each imported content after deserializing."""
         return obj
 
     def custom_obj_hook(self, obj, item):
@@ -483,7 +573,7 @@ class ImportContent(BrowserView):
 
             return self.portal['images']
         """
-        if self.request.get("import_to_current_folder", None):
+        if self.import_to_current_folder:
             return self.context
 
         container_path = self.CONTAINER.get(item["@type"], None)
@@ -545,7 +635,10 @@ class ImportContent(BrowserView):
         # so we need to encode parent_path before using plone.api.content.get
         if isinstance(self.context.getPhysicalPath()[0], bytes):
             parent_path = parent_path.encode("utf8")
-        parent = api.content.get(path=parent_path)
+        try:
+            parent = api.content.get(path=parent_path)
+        except NotFound:
+            parent = None
         if parent:
             # Check that we did not traverse to content outside of the portal.
             # Actually, we probably should not go outside the navigation root either,
@@ -602,8 +695,9 @@ class ImportContent(BrowserView):
 
     def set_uuid(self, item, obj):
         uuid = item["UID"]
-        if api.content.find(UID=uuid):
+        if not self.update_existing and api.content.find(UID=uuid):
             # this should only happen if you run import multiple times
+            # without updating existing content
             uuid = obj.UID()
             logger.info(
                 "UID {} of {} already in use by {}. Using {}".format(
@@ -648,7 +742,8 @@ class ResetModifiedAndCreatedDate(BrowserView):
                 obj.reindexObject(idxs=["created"])
 
         portal.ZopeFindAndApply(portal, search_sub=True, apply_func=reset_dates)
-        msg = "Finished resetting creation and modification date."
+        msg = "Finished resetting creation and modification dates."
+        logger.info(msg)
         api.portal.show_message(msg, self.request)
         return self.index()
 
