@@ -1,6 +1,7 @@
 # -*- coding: utf-8 -*-
 from Acquisition import aq_base
 from BTrees.LLBTree import LLSet
+from collective.exportimport import config
 from datetime import datetime
 from OFS.interfaces import IOrderedContainer
 from operator import itemgetter
@@ -13,7 +14,7 @@ from plone.portlets.interfaces import IPortletAssignmentSettings
 from plone.portlets.interfaces import ILocalPortletAssignmentManager
 from plone.portlets.interfaces import IPortletManager
 from Products.Five import BrowserView
-from six.moves.html_parser import HTMLParser
+from Products.ZCatalog.ProgressHandler import ZLogHandler
 from zope.annotation.interfaces import IAnnotations
 from zope.component import getUtility
 from zope.component import queryMultiAdapter
@@ -34,6 +35,15 @@ try:
 except ImportError:
     HAS_RELAPI = False
 
+
+try:
+    from Products.CMFPlone import relationhelper
+
+    HAS_PLONE6 = True
+except ImportError:
+    HAS_PLONE6 = False
+
+
 try:
     from plone.app.multilingual.interfaces import ITranslationManager
 
@@ -41,6 +51,13 @@ try:
 except ImportError:
     HAS_PAM = False
 
+if six.PY2:
+    from HTMLParser import HTMLParser
+    unescape = HTMLParser().unescape
+else:
+    from html import unescape
+
+
 logger = logging.getLogger(__name__)
 
 DISCUSSION_ANNOTATION_KEY = "plone.app.discussion:conversation"
@@ -246,9 +263,10 @@ class ImportRelations(BrowserView):
 
     def __call__(self, jsonfile=None, return_json=False):
 
-        if not HAS_RELAPI:
+        if not HAS_RELAPI and not HAS_PLONE6:
             api.portal.show_message(
-                "collective.relationhelpers is not available", self.request
+                "You need either Plone 6 or collective.relationhelpers to import relations",
+                self.request
             )
             return self.index()
 
@@ -311,9 +329,14 @@ class ImportRelations(BrowserView):
         all_fixed_relations = sorted(
             all_fixed_relations, key=itemgetter("from_uuid", "from_attribute")
         )
-        relapi.purge_relations()
-        relapi.cleanup_intids()
-        relapi.restore_relations(all_relations=all_fixed_relations)
+        if HAS_RELAPI:
+            relapi.purge_relations()
+            relapi.cleanup_intids()
+            relapi.restore_relations(all_relations=all_fixed_relations)
+        elif HAS_PLONE6:
+            relationhelper.purge_relations()
+            relationhelper.cleanup_intids()
+            relationhelper.restore_relations(all_relations=all_fixed_relations)
 
     def get_from_attribute(self, rel):
         # Optionally handle special cases...
@@ -356,7 +379,8 @@ class ImportLocalRoles(BrowserView):
 
     def import_localroles(self, data):
         results = 0
-        for item in data:
+        total = len(data)
+        for index, item in enumerate(data, start=1):
             obj = api.content.get(UID=item["uuid"])
             if not obj:
                 continue
@@ -364,17 +388,24 @@ class ImportLocalRoles(BrowserView):
                 localroles = item["localroles"]
                 for userid in localroles:
                     obj.manage_setLocalRoles(userid=userid, roles=localroles[userid])
-                logger.info(
+                logger.debug(
                     u"Set roles on {}: {}".format(obj.absolute_url(), localroles)
                 )
             if item.get("block"):
                 obj.__ac_local_roles_block__ = 1
-                logger.info(
+                logger.debug(
                     u"Diable acquisition of local roles on {}".format(
                         obj.absolute_url()
                     )
                 )
+            if not index % 1000:
+                logger.info(u"Set local roles on {} ({}%) of {} items".format(index, round(index / total * 100, 2), total))
             results += 1
+        if results:
+            logger.info("Reindexing Security")
+            catalog = api.portal.get_tool("portal_catalog")
+            pghandler = ZLogHandler(1000)
+            catalog.reindexIndex("allowedRolesAndUsers", None, pghandler=pghandler)
         return results
 
 
@@ -401,8 +432,12 @@ class ImportOrdering(BrowserView):
                     request=self.request,
                 )
             else:
+                start = datetime.now()
                 orders = self.import_ordering(data)
-                msg = u"Imported {} orders".format(orders)
+                end = datetime.now()
+                delta = end - start
+                msg = u"Imported {} orders in {} seconds".format(orders, delta.seconds)
+                logger.info(msg)
                 api.portal.show_message(msg, self.request)
             if return_json:
                 msg = {"state": status, "msg": msg}
@@ -412,7 +447,8 @@ class ImportOrdering(BrowserView):
 
     def import_ordering(self, data):
         results = 0
-        for item in data:
+        total = len(data)
+        for index, item in enumerate(data, start=1):
             obj = api.content.get(UID=item["uuid"])
             if not obj:
                 continue
@@ -420,6 +456,8 @@ class ImportOrdering(BrowserView):
             if not ordered:
                 continue
             ordered.moveObjectToPosition(obj.getId(), item["order"])
+            if not index % 1000:
+                logger.info(u"Ordered {} ({}%) of {} items".format(index, round(index / total * 100, 2), total))
             results += 1
         return results
 
@@ -461,13 +499,17 @@ class ImportDefaultPages(BrowserView):
         for item in data:
             obj = api.content.get(UID=item["uuid"])
             if not obj:
-                continue
+                if item["uuid"] == config.SITE_ROOT:
+                    obj = self.portal
+                else:
+                    continue
             old = obj.getDefaultPage()
             if six.PY2:
                 obj.setDefaultPage(item["default_page"].encode("utf-8"))
             else:
                 obj.setDefaultPage(item["default_page"])
             if old != obj.getDefaultPage():
+                logger.debug(u"Set {} as default page for {}".format(item["default_page"], obj.absolute_url()))
                 results += 1
         return results
 
@@ -506,7 +548,6 @@ class ImportDiscussion(BrowserView):
 
     def import_data(self, data):
         results = 0
-        html = HTMLParser()
         for conversation_data in data:
             obj = api.content.get(UID=conversation_data["uuid"])
             if not obj:
@@ -531,7 +572,7 @@ class ImportDiscussion(BrowserView):
                 comment.author_name = item["author_name"]
                 comment.author_username = item["author_username"]
                 comment.creator = item["author_username"]
-                comment.text = html.unescape(
+                comment.text = unescape(
                     item["text"]
                     .replace(u"\r<br />", u"\r\n")
                     .replace(u"<br />", u"\r\n")
